# -*- coding: utf-8 -*-
"""
    werkzeug.contrib
    ~~~~~~~~~~~~~~~~

    Contains user-submitted code that other users may find useful, but which
    is not part of the Werkzeug core.  Anyone can write code for inclusion in
    the `contrib` package.  All modules in this package are distributed as an
    add-on library and thus are not part of Werkzeug itself.

    This file itself is mostly for informational purposes and to tell the
    Python interpreter that `contrib` is a package.

    :copyright: (c) 2011 by the Werkzeug Team, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""
