{% extends "layout.html" %}
{% set page_title = 'About the Magic' %}
{% block page_body %}
  <p>
    Nothing to see. It's just magic.
  </p>
  <p>
    <a href="{{ h.url_for('static.index') }}">back to the index</a>
  </p>
{% endblock %}
